@@ -190,18 +190,32 @@ class DeleteVNFFG(command.Command):
             nargs="+",
             help=_("VNFFG(s) to delete (name or ID)")
         )
+        parser.add_argument(
+            '--force',
+            default=False,
+            action='store_true',
+            help=_('Force delete VNFFG'))
         return parser
 
+    def args2body(self, parsed_args):
+        body = dict()
+        if parsed_args.force:
+            body[_VNFFG] = dict()
+            body[_VNFFG]['attributes'] = dict()
+            body[_VNFFG]['attributes']['force'] = True
+        return body
+
     def take_action(self, parsed_args):
         client = self.app.client_manager.tackerclient
         failure = False
         deleted_ids = []
         failed_items = {}
+        body = self.args2body(parsed_args)
         for resource_id in parsed_args.vnffg:
             try:
                 obj = tackerV10.find_resourceid_by_name_or_id(
                     client, _VNFFG, resource_id)
-                client.delete_vnffg(obj)
+                client.delete_vnffg(obj, body)
                 deleted_ids.append(resource_id)
             except Exception as e:
                 failure = True
