@@ -41,7 +41,8 @@ LOG = logging.getLogger(__name__)
 _mixed_case_fields = ('vnfInstanceName', 'vnfInstanceDescription', 'vnfdId',
                       'vnfProvider', 'vnfProductName', 'vnfSoftwareVersion',
                       'vnfdVersion', 'instantiationState',
-                      'vimConnectionInfo', 'instantiatedVnfInfo')
+                      'vimConnectionInfo', 'instantiatedVnfInfo',
+                      'vnfConfigurableProperties')
 
 _VNF_INSTANCE = 'vnf_instance'
 
@@ -68,6 +69,7 @@ def _get_columns(vnflcm_obj, action=None):
         'vnfdVersion': 'VNFD Version',
         'instantiationState': 'Instantiation State',
         '_links': 'Links',
+        'vnfConfigurableProperties': 'VNF Configurable Properties',
     }
     if action == 'show':
         if vnflcm_obj['instantiationState'] == 'INSTANTIATED':
@@ -236,6 +238,12 @@ class HealVnfLcm(command.Command):
 
     def get_parser(self, prog_name):
         parser = super(HealVnfLcm, self).get_parser(prog_name)
+        usage_message = ('''%(prog)s [-h] [--cause CAUSE]
+                             [--vnfc-instance <vnfc-instance-id> '''
+                         '''[<vnfc-instance-id> ...]]
+                             [--additional-param-file <additional-param-file>]
+                             -- <vnf-instance>''')
+        parser.usage = usage_message
         parser.add_argument(
             _VNF_INSTANCE,
             metavar="<vnf-instance>",
@@ -249,6 +257,11 @@ class HealVnfLcm(command.Command):
             nargs="+",
             help=_("List of VNFC instances requiring a healing action.")
         )
+        parser.add_argument(
+            '--additional-param-file',
+            metavar="<additional-param-file>",
+            help=_("Additional parameters passed by the NFVO as input "
+                   "to the healing process."))
         return parser
 
     def args2body(self, parsed_args):
@@ -257,6 +270,8 @@ class HealVnfLcm(command.Command):
             body['cause'] = parsed_args.cause
         if parsed_args.vnfc_instance:
             body['vnfcInstanceId'] = parsed_args.vnfc_instance
+        if parsed_args.additional_param_file:
+            body.update(jsonfile2body(parsed_args.additional_param_file))
 
         return body
 
@@ -332,7 +347,7 @@ class TerminateVnfLcm(command.Command):
 
                 result = client.delete_vnf_instance(parsed_args.vnf_instance)
                 if not result:
-                    print(_("VNF Instance '%(id)s' deleted successfully") %
+                    print(_("VNF Instance '%(id)s' is deleted successfully") %
                           {'id': parsed_args.vnf_instance})
 
     def _wait_until_vnf_is_terminated(self, client, vnf_instance_id,
@@ -405,7 +420,7 @@ class DeleteVnfLcm(command.Command):
                 print(_('All specified vnf instances are deleted '
                         'successfully'))
             else:
-                print(_("Vnf instance '%s' deleted "
+                print(_("Vnf instance '%s' is deleted "
                         "successfully") % vnf_instances[0])
 
 
@@ -476,46 +491,51 @@ class ScaleVnfLcm(command.Command):
             _VNF_INSTANCE,
             metavar="<vnf-instance>",
             help=_('VNF instance ID to scale'))
-        parser.add_argument(
-            '--I',
-            metavar="<param-file>",
-            help=_("Specify scale request parameters in a json file."))
-        parser.add_argument(
-            '--type',
-            metavar="<type>",
-            choices=['SCALE_OUT', 'SCALE_IN'],
-            help=_("Indicates the type of the scale operation requested"))
-        parser.add_argument(
-            '--aspect-id',
-            metavar="<aspect-id>",
-            help=_("Identifier of the scaling aspect."))
         parser.add_argument(
             '--number-of-steps',
             metavar="<number-of-steps>",
             type=int,
-            help=_("Number of scaling steps to be executed as part of"
+            help=_("Number of scaling steps to be executed as part of "
                    "this Scale VNF operation."))
         parser.add_argument(
             '--additional-param-file',
             metavar="<additional-param-file>",
-            help=_("Additional parameters passed by the NFVO as input"
+            help=_("Additional parameters passed by the NFVO as input "
                    "to the scaling process."))
+
+        scale_require_parameters = parser.add_argument_group(
+            "require arguments"
+        )
+        scale_require_parameters.add_argument(
+            '--type',
+            metavar="<type>",
+            required=True,
+            choices=['SCALE_OUT', 'SCALE_IN'],
+            help=_("SCALE_OUT or SCALE_IN for type of scale operation."))
+        scale_require_parameters.add_argument(
+            '--aspect-id',
+            required=True,
+            metavar="<aspect-id>",
+            help=_("Identifier of the scaling aspect."))
+
         return parser
 
-    def args2body(self, file_path=None):
+    def args2body(self, parsed_args):
         """To store request body, call jsonfile2body.
 
         Args:
-            file_path ([string], optional): file path of param file(json).
-            Defaults to None.
+            parsed_args ([Namespace]): arguments of CLI.
 
         Returns:
-            body[dict]: [description]
+            body ([dict]): Request body is stored
         """
-        body = {}
+        body = {'type': parsed_args.type, 'aspectId': parsed_args.aspect_id}
 
-        if file_path:
-            return jsonfile2body(file_path)
+        if parsed_args.number_of_steps:
+            body['numberOfSteps'] = parsed_args.number_of_steps
+
+        if parsed_args.additional_param_file:
+            body.update(jsonfile2body(parsed_args.additional_param_file))
 
         return body
 
@@ -523,13 +543,66 @@ class ScaleVnfLcm(command.Command):
         """Execute scale_vnf_instance and output result comment.
 
         Args:
-            parsed_args ([Namespace]): [description]
+            parsed_args ([Namespace]): arguments of CLI.
         """
         client = self.app.client_manager.tackerclient
-        if parsed_args.additional_param_file:
-            result = client.scale_vnf_instance(
-                parsed_args.vnf_instance,
-                self.args2body(file_path=parsed_args.additional_param_file))
-            if not result:
-                print((_('Scale request for VNF Instance %(id)s has been'
-                         ' accepted.') % {'id': parsed_args.vnf_instance}))
+        result = client.scale_vnf_instance(
+            parsed_args.vnf_instance,
+            self.args2body(parsed_args))
+        if not result:
+            print((_('Scale request for VNF Instance %s has been accepted.')
+                   % parsed_args.vnf_instance))
+
+
+class ChangeExtConnVnfLcm(command.Command):
+    _description = _("Change External VNF Connectivity")
+
+    def get_parser(self, prog_name):
+        parser = super(ChangeExtConnVnfLcm, self).get_parser(prog_name)
+        parser.add_argument(
+            _VNF_INSTANCE,
+            metavar="<vnf-instance>",
+            help=_("VNF instance ID to Change External VNF Connectivity"))
+        parser.add_argument(
+            'request_file',
+            metavar="<param-file>",
+            help=_("Specify change-ext-conn request parameters "
+                   "in a json file."))
+
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.tackerclient
+        result = client.change_ext_conn_vnf_instance(
+            parsed_args.vnf_instance, jsonfile2body(
+                parsed_args.request_file))
+        if not result:
+            print((_('Change External VNF Connectivity for VNF Instance %s '
+                     'has been accepted.') % parsed_args.vnf_instance))
+
+
+class ChangeVnfPkgVnfLcm(command.Command):
+    _description = _("Change Current VNF Package")
+
+    def get_parser(self, prog_name):
+        parser = super(ChangeVnfPkgVnfLcm, self).get_parser(prog_name)
+        parser.add_argument(
+            _VNF_INSTANCE,
+            metavar="<vnf-instance>",
+            help=_("VNF instance ID to Change Current VNF Package"))
+        parser.add_argument(
+            'request_file',
+            metavar="<param-file>",
+            help=_("Specify change-vnfpkg request parameters "
+                   "in a json file."))
+
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.tackerclient
+        result = client.change_vnfpkg_vnf_instance(
+            parsed_args.vnf_instance, jsonfile2body(
+                parsed_args.request_file))
+        if not result:
+            print((_('Change Current VNF Package for VNF Instance %s '
+                     'has been accepted.') % parsed_args.vnf_instance))
