@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from io import StringIO
 import os
 import sys
@@ -20,7 +21,6 @@ from unittest import mock
 
 import ddt
 from oslo_utils.fixture import uuidsentinel
-import six
 
 from tackerclient.common import exceptions
 from tackerclient.osc import utils as tacker_osc_utils
@@ -47,12 +47,14 @@ class TestVnfLcm(base.FixturedTestCase):
 def _get_columns_vnflcm(action='create'):
     columns = ['ID', 'Instantiation State', 'VNF Instance Description',
                'VNF Instance Name', 'VNF Product Name', 'VNF Provider',
-               'VNF Software Version', 'VNFD ID', 'VNFD Version', 'Links']
+               'VNF Software Version', 'VNFD ID', 'VNFD Version', 'Links',
+               'VNF Configurable Properties']
     if action == 'show':
         columns.extend(['Instantiated Vnf Info', 'VIM Connection Info'])
     if action == 'list':
         columns = [ele for ele in columns if ele not in
-                   ['VNFD Version', 'VNF Instance Description']]
+                   ['VNFD Version', 'VNF Instance Description',
+                    'VNF Configurable Properties']]
         columns.remove('Links')
     return columns
 
@@ -181,6 +183,48 @@ class TestListVnfLcm(TestVnfLcm):
                               actual_columns)
         self.assertCountEqual(expected_data, list(data))
 
+    def test_take_action_with_pagination(self):
+        next_links_num = 3
+        parsed_args = self.check_parser(self.list_vnf_instance, [], [])
+        path = os.path.join(self.url, 'vnflcm/v1/vnf_instances')
+
+        links = [0] * next_links_num
+        link_headers = [0] * next_links_num
+
+        for i in range(next_links_num):
+            links[i] = (
+                '{base_url}?nextpage_opaque_marker={vnf_instance_id}'.format(
+                    base_url=path,
+                    vnf_instance_id=self.vnf_instances[i]['id']))
+            link_headers[i] = copy.deepcopy(self.header)
+            link_headers[i]['Link'] = '<{link_url}>; rel="next"'.format(
+                link_url=links[i])
+
+        self.requests_mock.register_uri(
+            'GET', path, json=[self.vnf_instances[0]], headers=link_headers[0])
+        self.requests_mock.register_uri(
+            'GET', links[0], json=[self.vnf_instances[1]],
+            headers=link_headers[1])
+        self.requests_mock.register_uri(
+            'GET', links[1], json=[self.vnf_instances[2]],
+            headers=link_headers[2])
+        self.requests_mock.register_uri(
+            'GET', links[2], json=[], headers=self.header)
+
+        actual_columns, data = self.list_vnf_instance.take_action(parsed_args)
+
+        headers, columns = tacker_osc_utils.get_column_definitions(
+            vnflcm._attr_map, long_listing=True)
+
+        expected_data = []
+        for vnf_instance_obj in self.vnf_instances:
+            expected_data.append(vnflcm_fakes.get_vnflcm_data(
+                vnf_instance_obj, columns=columns, list_action=True))
+
+        self.assertCountEqual(_get_columns_vnflcm(action='list'),
+                              actual_columns)
+        self.assertCountEqual(expected_data, list(data))
+
 
 class TestInstantiateVnfLcm(TestVnfLcm):
 
@@ -287,6 +331,9 @@ class TestHealVnfLcm(TestVnfLcm):
         self.heal_vnf_lcm = vnflcm.HealVnfLcm(
             self.app, self.app_args, cmd_name='vnflcm heal')
 
+    _heal_sample_param_file = ("./tackerclient/osc/v1/vnflcm/samples/"
+                               "heal_vnf_instance_param_sample.json")
+
     @ddt.data((['--cause', 'test-cause', "--vnfc-instance",
                 'vnfc-id-1', 'vnfc-id-2'],
                [('cause', 'test-cause'),
@@ -295,6 +342,8 @@ class TestHealVnfLcm(TestVnfLcm):
                [('cause', 'test-cause')]),
               (["--vnfc-instance", 'vnfc-id-1', 'vnfc-id-2'],
                [('vnfc_instance', ['vnfc-id-1', 'vnfc-id-2'])]),
+              (["--additional-param-file", _heal_sample_param_file],
+               [('additional_param_file', _heal_sample_param_file)]),
               ([], []))
     @ddt.unpack
     def test_take_action(self, arglist, verifylist):
@@ -339,6 +388,25 @@ class TestHealVnfLcm(TestVnfLcm):
                           self.heal_vnf_lcm.take_action,
                           parsed_args)
 
+    def test_take_action_param_file_not_exists(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = "./not_exists.json"
+        arglist = [vnf_instance['id'],
+                   '--additional-param-file', sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('additional_param_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(self.heal_vnf_lcm, arglist,
+                                        verifylist)
+
+        ex = self.assertRaises(exceptions.InvalidInput,
+                               self.heal_vnf_lcm.take_action, parsed_args)
+
+        expected_msg = ("Invalid input: File %s does not exist "
+                        "or user does not have read privileges to it")
+        self.assertEqual(expected_msg % sample_param_file, str(ex))
+
 
 @ddt.ddt
 class TestTerminateVnfLcm(TestVnfLcm):
@@ -395,7 +463,7 @@ class TestTerminateVnfLcm(TestVnfLcm):
             self.assertIn(expected_message, actual_message)
 
             if delete_vnf:
-                expected_message = ("VNF Instance '%s' deleted successfully"
+                expected_message = ("VNF Instance '%s' is deleted successfully"
                                     % vnf_instance['id'])
                 self.assertIn(expected_message, actual_message)
 
@@ -448,7 +516,7 @@ class TestTerminateVnfLcm(TestVnfLcm):
                                 "delete vnf instance %(id)s"
                                 % {'timeout': 15, 'id': vnf_instance['id']})
 
-            self.assertIn(expected_message, six.text_type(result))
+            self.assertIn(expected_message, str(result))
             self.assertNotCalled(mock_delete)
 
     def test_terminate_no_options(self):
@@ -511,7 +579,7 @@ class TestDeleteVnfLcm(TestVnfLcm):
         sys.stdout = buffer = StringIO()
         result = self.delete_vnf_instance.take_action(parsed_args)
         self.assertIsNone(result)
-        self.assertEqual(("Vnf instance '%s' deleted successfully")
+        self.assertEqual(("Vnf instance '%s' is deleted successfully")
                          % self.vnf_instances[0]['id'],
                          buffer.getvalue().strip())
 
@@ -652,6 +720,38 @@ class TestScaleVnfLcm(TestVnfLcm):
 
         self.assertEqual(expected_message, actual_message)
 
+    @ddt.data('SCALE_IN', 'SCALE_OUT')
+    def test_take_action_no_param_file(self, scale_type):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+
+        arglist = [vnf_instance['id'],
+                   '--aspect-id', uuidsentinel.aspect_id,
+                   '--number-of-steps', '1',
+                   '--type', scale_type]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('aspect_id', uuidsentinel.aspect_id),
+                      ('number_of_steps', 1),
+                      ('type', scale_type)]
+
+        parsed_args = self.check_parser(self.scale_vnf_lcm, arglist,
+                                        verifylist)
+
+        url = os.path.join(self.url, 'vnflcm/v1/vnf_instances',
+                           vnf_instance['id'], 'scale')
+
+        self.requests_mock.register_uri(
+            'POST', url, headers=self.header, json={})
+
+        sys.stdout = buffer = StringIO()
+        self.scale_vnf_lcm.take_action(parsed_args)
+
+        actual_message = buffer.getvalue().strip()
+
+        expected_message = ("Scale request for VNF Instance %s has been "
+                            "accepted.") % vnf_instance['id']
+
+        self.assertEqual(expected_message, actual_message)
+
     @ddt.data('SCALE_IN', 'SCALE_OUT')
     def test_take_action_param_file_not_exists(self, scale_type):
         vnf_instance = vnflcm_fakes.vnf_instance_response()
@@ -708,3 +808,159 @@ class TestScaleVnfLcm(TestVnfLcm):
         self.assertRaises(exceptions.TackerClientException,
                           self.scale_vnf_lcm.take_action,
                           parsed_args)
+
+
+class TestChangeExtConnVnfLcm(TestVnfLcm):
+
+    def setUp(self):
+        super(TestChangeExtConnVnfLcm, self).setUp()
+        self.change_ext_conn_vnf_lcm = vnflcm.ChangeExtConnVnfLcm(
+            self.app, self.app_args,
+            cmd_name='vnflcm change-ext-conn')
+
+    def test_take_action(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = ("./tackerclient/osc/v1/vnflcm/samples/"
+                             "change_ext_conn_vnf_instance_param_sample.json")
+
+        arglist = [vnf_instance['id'], sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('request_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(self.change_ext_conn_vnf_lcm,
+                                        arglist,
+                                        verifylist)
+
+        url = os.path.join(self.url, 'vnflcm/v1/vnf_instances',
+                           vnf_instance['id'], 'change_ext_conn')
+        self.requests_mock.register_uri(
+            'POST', url, headers=self.header, json={})
+
+        sys.stdout = buffer = StringIO()
+        with mock.patch.object(proxy_client.ClientBase,
+                               '_handle_fault_response') as m:
+            self.change_ext_conn_vnf_lcm.take_action(parsed_args)
+            # check no fault response is received
+            self.assertNotCalled(m)
+            self.assertEqual(
+                ('Change External VNF Connectivity for VNF Instance {0} '
+                 'has been accepted.'.format(vnf_instance['id'])),
+                buffer.getvalue().strip())
+
+    def test_take_action_vnf_instance_not_found(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = ("./tackerclient/osc/v1/vnflcm/samples/"
+                             "change_ext_conn_vnf_instance_param_sample.json")
+        arglist = [vnf_instance['id'], sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('request_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(self.change_ext_conn_vnf_lcm,
+                                        arglist,
+                                        verifylist)
+
+        url = os.path.join(self.url, 'vnflcm/v1/vnf_instances',
+                           vnf_instance['id'], 'change_ext_conn')
+        self.requests_mock.register_uri(
+            'POST', url, headers=self.header, status_code=404, json={})
+
+        self.assertRaises(exceptions.TackerClientException,
+                          self.change_ext_conn_vnf_lcm.take_action,
+                          parsed_args)
+
+    def test_take_action_param_file_not_exists(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = "./not_exists.json"
+        arglist = [vnf_instance['id'], sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('request_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(
+            self.change_ext_conn_vnf_lcm,
+            arglist,
+            verifylist)
+
+        ex = self.assertRaises(
+            exceptions.InvalidInput,
+            self.change_ext_conn_vnf_lcm.take_action,
+            parsed_args)
+
+        expected_msg = ("Invalid input: File %s does not exist "
+                        "or user does not have read privileges to it")
+        self.assertEqual(expected_msg % sample_param_file, str(ex))
+
+    @mock.patch("os.open")
+    @mock.patch("os.access")
+    def test_take_action_invalid_format_param_file(self, mock_open,
+                                                   mock_access):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = "./invalid_param_file.json"
+        arglist = [vnf_instance['id'], sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('request_file', sample_param_file)]
+
+        mock_open.return_value = "invalid_json_data"
+        # command param
+        parsed_args = self.check_parser(self.change_ext_conn_vnf_lcm,
+                                        arglist,
+                                        verifylist)
+
+        ex = self.assertRaises(
+            exceptions.InvalidInput,
+            self.change_ext_conn_vnf_lcm.take_action,
+            parsed_args)
+
+        expected_msg = "Failed to load parameter file."
+        self.assertIn(expected_msg, str(ex))
+
+
+class TestChangeVnfPkgVnfLcm(TestVnfLcm):
+
+    def setUp(self):
+        super(TestChangeVnfPkgVnfLcm, self).setUp()
+        self.change_vnfpkg_vnf_lcm = vnflcm.ChangeVnfPkgVnfLcm(
+            self.app, self.app_args,
+            cmd_name='vnflcm change-vnfpkg')
+
+    def test_take_action_with_v1_version(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = ("./tackerclient/osc/v2/vnflcm/samples/"
+                             "change_vnfpkg_vnf_instance_param_sample.json")
+        arglist = [vnf_instance['id'], sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('request_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(self.change_vnfpkg_vnf_lcm,
+                                        arglist,
+                                        verifylist)
+
+        url = os.path.join(self.url, 'vnflcm/v1/vnf_instances',
+                           vnf_instance['id'], 'change_vnfpkg')
+        self.requests_mock.register_uri(
+            'POST', url, headers=self.header, status_code=400, json={})
+
+        ex = self.assertRaises(exceptions.UnsupportedCommandVersion,
+                               self.change_vnfpkg_vnf_lcm.take_action,
+                               parsed_args)
+
+        expected_msg = "This command is not supported in version 1"
+        self.assertEqual(expected_msg, str(ex))
+
+
+class TestVnfLcmV1(base.FixturedTestCase):
+    client_fixture_class = client.ClientFixture
+    api_version = '1'
+
+    def setUp(self):
+        super(TestVnfLcmV1, self).setUp()
+
+    def test_client_v2(self):
+        self.assertEqual(self.cs.vnf_lcm_client.headers,
+                         {'Version': '1.3.0'})
+        self.assertEqual(self.cs.vnf_lcm_client.vnf_instances_path,
+                         '/vnflcm/v1/vnf_instances')
+        # check of other paths is omitted.
