@@ -14,18 +14,17 @@
 #    under the License.
 
 import argparse
+import fixtures
+import io
 import logging
 import os
 import re
-from unittest import mock
-
-import six
 import sys
-
-import fixtures
-from keystoneclient import session
 import testtools
 from testtools import matchers
+from unittest import mock
+
+from keystoneclient import session
 
 from tackerclient.common import clientmanager
 from tackerclient import shell as openstack_shell
@@ -63,8 +62,8 @@ class ShellTest(testtools.TestCase):
         clean_env = {}
         _old_env, os.environ = os.environ, clean_env.copy()
         try:
-            sys.stdout = six.StringIO()
-            sys.stderr = six.StringIO()
+            sys.stdout = io.StringIO()
+            sys.stderr = io.StringIO()
             _shell = openstack_shell.TackerShell(DEFAULT_API_VERSION)
             _shell.run(argstr.split())
         except SystemExit:
@@ -155,10 +154,10 @@ class ShellTest(testtools.TestCase):
     @mock.patch.object(openstack_shell.TackerShell, 'run')
     def test_main_with_unicode(self, mock_run):
         mock_run.return_value = 0
-        unicode_text = u'\u7f51\u7edc'
+        unicode_text = '\u7f51\u7edc'
         argv = ['net-list', unicode_text, unicode_text.encode('utf-8')]
         ret = openstack_shell.main(argv=argv)
-        mock_run.assert_called_once_with([u'net-list', unicode_text,
+        mock_run.assert_called_once_with(['net-list', unicode_text,
                                           unicode_text])
         self.assertEqual(0, ret)
 
