@@ -16,10 +16,11 @@
 #
 
 import logging
+import re
 import time
 
 import requests
-import six.moves.urllib.parse as urlparse
+from urllib import parse as urlparse
 
 from tackerclient import client
 from tackerclient.common import constants
@@ -54,6 +55,7 @@ def exception_handler_v10(status_code, error_content):
     :param status_code: HTTP error status code
     :param error_content: deserialized body of error response
     """
+    etsi_error_content = error_content
     error_dict = None
     if isinstance(error_content, dict):
         error_dict = error_content.get('TackerError')
@@ -94,6 +96,13 @@ def exception_handler_v10(status_code, error_content):
         if message:
             raise exceptions.TackerClientException(status_code=status_code,
                                                    message=message)
+        # ETSI error response
+        if isinstance(etsi_error_content, dict):
+            if etsi_error_content.get('status') and \
+                etsi_error_content.get('detail'):
+                message = etsi_error_content.get('detail')
+                raise exceptions.TackerClientException(status_code=status_code,
+                                                       message=message)
 
     # If we end up here the exception was not a tacker error
     msg = "%s-%s" % (status_code, error_content)
@@ -172,6 +181,8 @@ class ClientBase(object):
         self.format = 'json'
         self.action_prefix = "/v%s" % (self.version)
         self.retry_interval = 1
+        self.rel = None
+        self.params = None
 
     def _handle_fault_response(self, status_code, response_body):
         # Create exception with HTTP status code and message
@@ -221,8 +232,12 @@ class ClientBase(object):
         if body or body == {}:
             body = self.serialize(body)
 
+        if headers is None:
+            # self.httpclient.do_request is not accept 'headers=None'.
+            headers = {}
+
         resp, replybody = self.httpclient.do_request(
-            action, method, body=body,
+            action, method, body=body, headers=headers,
             content_type=self.content_type())
 
         if 'application/zip' == resp.headers.get('Content-Type'):
@@ -234,6 +249,19 @@ class ClientBase(object):
         else:
             self.format = 'json'
 
+        url = None
+        rel = None
+
+        link = resp.headers.get('Link', None)
+        if link is not None:
+            url = re.findall('<(.*)>', link)[0]
+            rel = re.findall('rel="(.*)"', link)[0]
+
+        if rel == 'next':
+            self.rel = 'next'
+            query_str = urlparse.urlparse(url).query
+            self.params = urlparse.parse_qs(query_str)
+
         status_code = resp.status_code
         if status_code in (requests.codes.ok,
                            requests.codes.created,
@@ -347,40 +375,43 @@ class ClientBase(object):
         return self.retry_request("PATCH", action, body=body,
                                   headers=headers, params=params)
 
-    def list(self, collection, path, retrieve_all=True, **params):
+    def list(self, collection, path, retrieve_all=True, headers=None,
+             **params):
         if retrieve_all:
             res = []
-            for r in self._pagination(collection, path, **params):
+            for r in self._pagination(collection, path, headers, **params):
                 if type(r) is list:
                     res.extend(r)
                 else:
                     res.extend(r[collection])
             return {collection: res} if collection else res
         else:
-            return self._pagination(collection, path, **params)
+            return self._pagination(collection, path, headers, **params)
 
-    def _pagination(self, collection, path, **params):
+    def _pagination(self, collection, path, headers, **params):
         if params.get('page_reverse', False):
             linkrel = 'previous'
         else:
             linkrel = 'next'
         next = True
         while next:
-            res = self.get(path, params=params)
+            self.rel = None
+            res = self.get(path, headers=headers, params=params)
             yield res
             next = False
             try:
-                # TODO(tpatil): Handle pagination for list data type
-                # once it's supported by tacker.
                 if type(res) is list:
-                    break
-
-                for link in res['%s_links' % collection]:
-                    if link['rel'] == linkrel:
-                        query_str = urlparse.urlparse(link['href']).query
-                        params = urlparse.parse_qs(query_str)
+                    if self.rel == 'next':
+                        params = self.params
                         next = True
-                        break
+
+                else:
+                    for link in res['%s_links' % collection]:
+                        if link['rel'] == linkrel:
+                            query_str = urlparse.urlparse(link['href']).query
+                            params = urlparse.parse_qs(query_str)
+                            next = True
+                            break
             except KeyError:
                 break
 
@@ -618,8 +649,8 @@ class LegacyClient(ClientBase):
         return self.post(self.vnffgs_path, body=body)
 
     @APIParamsCall
-    def delete_vnffg(self, vnffg):
-        return self.delete(self.vnffg_path % vnffg)
+    def delete_vnffg(self, vnffg, body=None):
+        return self.delete(self.vnffg_path % vnffg, body=body)
 
     @APIParamsCall
     def update_vnffg(self, vnffg, body):
@@ -866,58 +897,251 @@ class VnfLCMClient(ClientBase):
     APIs.
     """
 
-    vnf_instances_path = '/vnflcm/v1/vnf_instances'
-    vnf_instance_path = '/vnflcm/v1/vnf_instances/%s'
-    vnf_lcm_op_occs_path = '/vnflcm/v1/vnf_lcm_op_occs/%s'
+    def __init__(self, api_version, **kwargs):
+        super(VnfLCMClient, self).__init__(**kwargs)
+        self.headers = {'Version': '1.3.0'}
+        sol_api_version = 'v1'
+        if api_version == '2':
+            self.headers = {'Version': '2.0.0'}
+            sol_api_version = 'v2'
+
+        self.vnf_instances_path = (
+            '/vnflcm/{}/vnf_instances'.format(sol_api_version))
+        self.vnf_instance_path = (
+            '/vnflcm/{}/vnf_instances/%s'.format(sol_api_version))
+        self.vnf_lcm_op_occurrences_path = (
+            '/vnflcm/{}/vnf_lcm_op_occs'.format(sol_api_version))
+        self.vnf_lcm_op_occs_path = (
+            '/vnflcm/{}/vnf_lcm_op_occs/%s'.format(sol_api_version))
+        self.lccn_subscriptions_path = (
+            '/vnflcm/{}/subscriptions'.format(sol_api_version))
+        self.lccn_subscription_path = (
+            '/vnflcm/{}/subscriptions/%s'.format(sol_api_version))
 
     def build_action(self, action):
         return action
 
     @APIParamsCall
     def create_vnf_instance(self, body):
-        return self.post(self.vnf_instances_path, body=body)
+        return self.post(self.vnf_instances_path, body=body,
+                         headers=self.headers)
 
     @APIParamsCall
     def show_vnf_instance(self, vnf_id, **_params):
-        return self.get(self.vnf_instance_path % vnf_id, params=_params)
+        return self.get(self.vnf_instance_path % vnf_id,
+                        headers=self.headers, params=_params)
 
     @APIParamsCall
     def list_vnf_instances(self, retrieve_all=True, **_params):
         vnf_instances = self.list(None, self.vnf_instances_path,
-                                  retrieve_all, **_params)
+                                  retrieve_all, headers=self.headers,
+                                  **_params)
         return vnf_instances
 
     @APIParamsCall
     def instantiate_vnf_instance(self, vnf_id, body):
         return self.post((self.vnf_instance_path + "/instantiate") % vnf_id,
-                         body=body)
+                         body=body, headers=self.headers)
 
     @APIParamsCall
     def heal_vnf_instance(self, vnf_id, body):
         return self.post((self.vnf_instance_path + "/heal") % vnf_id,
-                         body=body)
+                         body=body, headers=self.headers)
 
     @APIParamsCall
     def terminate_vnf_instance(self, vnf_id, body):
         return self.post((self.vnf_instance_path + "/terminate") % vnf_id,
-                         body=body)
+                         body=body, headers=self.headers)
 
     @APIParamsCall
     def delete_vnf_instance(self, vnf_id):
-        return self.delete(self.vnf_instance_path % vnf_id)
+        return self.delete(self.vnf_instance_path % vnf_id,
+                           headers=self.headers)
 
     @APIParamsCall
     def update_vnf_instance(self, vnf_id, body):
-        return self.patch(self.vnf_instance_path % vnf_id, body=body)
+        return self.patch(self.vnf_instance_path % vnf_id, body=body,
+                          headers=self.headers)
 
     @APIParamsCall
     def scale_vnf_instance(self, vnf_id, body):
         return self.post((self.vnf_instance_path + "/scale") % vnf_id,
-                         body=body)
+                         body=body, headers=self.headers)
 
     @APIParamsCall
     def rollback_vnf_instance(self, occ_id):
-        return self.post((self.vnf_lcm_op_occs_path + "/rollback") % occ_id)
+        return self.post((self.vnf_lcm_op_occs_path + "/rollback") % occ_id,
+                         headers=self.headers)
+
+    @APIParamsCall
+    def cancel_vnf_instance(self, occ_id, body):
+        return self.post((self.vnf_lcm_op_occs_path + "/cancel") % occ_id,
+                         body=body)
+
+    @APIParamsCall
+    def fail_vnf_instance(self, occ_id):
+        return self.post((self.vnf_lcm_op_occs_path + "/fail") % occ_id,
+                         headers=self.headers)
+
+    @APIParamsCall
+    def change_ext_conn_vnf_instance(self, vnf_id, body):
+        return self.post((self.vnf_instance_path + "/change_ext_conn") %
+                         vnf_id, body=body, headers=self.headers)
+
+    @APIParamsCall
+    def change_vnfpkg_vnf_instance(self, vnf_id, body):
+        # NOTE: it is only supported by V2-API.
+        if self.vnf_instance_path.split('/')[2] == 'v2':
+            return self.post((self.vnf_instance_path + "/change_vnfpkg") %
+                             vnf_id, body=body, headers=self.headers)
+        else:
+            raise exceptions.UnsupportedCommandVersion(version='1')
+
+    @APIParamsCall
+    def retry_vnf_instance(self, occ_id):
+        return self.post((self.vnf_lcm_op_occs_path + "/retry") % occ_id,
+                         headers=self.headers)
+
+    @APIParamsCall
+    def list_vnf_lcm_op_occs(self, retrieve_all=True, **_params):
+        vnf_lcm_op_occs = self.list(None, self.vnf_lcm_op_occurrences_path,
+                                    retrieve_all, headers=self.headers,
+                                    **_params)
+        return vnf_lcm_op_occs
+
+    @APIParamsCall
+    def show_vnf_lcm_op_occs(self, occ_id):
+        return self.get(self.vnf_lcm_op_occs_path % occ_id,
+                        headers=self.headers)
+
+    @APIParamsCall
+    def create_lccn_subscription(self, body):
+        return self.post(self.lccn_subscriptions_path, body=body,
+                         headers=self.headers)
+
+    @APIParamsCall
+    def delete_lccn_subscription(self, subsc_id):
+        return self.delete(self.lccn_subscription_path % subsc_id,
+                           headers=self.headers)
+
+    @APIParamsCall
+    def list_lccn_subscriptions(self, retrieve_all=True, **_params):
+        subscriptions = self.list(None, self.lccn_subscriptions_path,
+                                  retrieve_all, headers=self.headers,
+                                  **_params)
+        return subscriptions
+
+    @APIParamsCall
+    def show_lccn_subscription(self, subsc_id):
+        return self.get(self.lccn_subscription_path % subsc_id,
+                        headers=self.headers)
+
+    @APIParamsCall
+    def show_vnf_lcm_versions(self, major_version):
+        if major_version is None:
+            path = "/vnflcm/api_versions"
+        else:
+            path = "/vnflcm/{}/api_versions".format(major_version)
+        # NOTE: This may be called with any combination of
+        # --os-tacker-api-verson:[1, 2] and major_version:[None, 1, 2].
+        # Specifying "headers={'Version': '2.0.0'}" is most simple to
+        # make all cases OK.
+        return self.get(path, headers={'Version': '2.0.0'})
+
+
+class VnfFMClient(ClientBase):
+    headers = {'Version': '1.3.0'}
+    vnf_fm_alarms_path = '/vnffm/v1/alarms'
+    vnf_fm_alarm_path = '/vnffm/v1/alarms/%s'
+    vnf_fm_subs_path = '/vnffm/v1/subscriptions'
+    vnf_fm_sub_path = '/vnffm/v1/subscriptions/%s'
+
+    def build_action(self, action):
+        return action
+
+    @APIParamsCall
+    def list_vnf_fm_alarms(self, retrieve_all=True, **_params):
+        vnf_fm_alarms = self.list(
+            "vnf_fm_alarms", self.vnf_fm_alarms_path, retrieve_all,
+            headers=self.headers, **_params)
+        return vnf_fm_alarms
+
+    @APIParamsCall
+    def show_vnf_fm_alarm(self, vnf_fm_alarm_id):
+        return self.get(
+            self.vnf_fm_alarm_path % vnf_fm_alarm_id, headers=self.headers)
+
+    @APIParamsCall
+    def update_vnf_fm_alarm(self, vnf_fm_alarm_id, body):
+        return self.patch(
+            self.vnf_fm_alarm_path % vnf_fm_alarm_id, body=body,
+            headers=self.headers)
+
+    @APIParamsCall
+    def create_vnf_fm_sub(self, body):
+        return self.post(
+            self.vnf_fm_subs_path, body=body, headers=self.headers)
+
+    @APIParamsCall
+    def list_vnf_fm_subs(self, retrieve_all=True, **_params):
+        vnf_fm_subs = self.list("vnf_fm_subs", self.vnf_fm_subs_path,
+                                retrieve_all, headers=self.headers, **_params)
+        return vnf_fm_subs
+
+    @APIParamsCall
+    def show_vnf_fm_sub(self, vnf_fm_sub_id):
+        return self.get(
+            self.vnf_fm_sub_path % vnf_fm_sub_id, headers=self.headers)
+
+    @APIParamsCall
+    def delete_vnf_fm_sub(self, vnf_fm_sub_id):
+        return self.delete(
+            self.vnf_fm_sub_path % vnf_fm_sub_id, headers=self.headers)
+
+
+class VnfPMClient(ClientBase):
+    headers = {'Version': '2.1.0'}
+    vnf_pm_jobs_path = '/vnfpm/v2/pm_jobs'
+    vnf_pm_job_path = '/vnfpm/v2/pm_jobs/%s'
+    vnf_pm_reports_path = '/vnfpm/v2/pm_jobs/%(job_id)s/reports/%(report_id)s'
+
+    def build_action(self, action):
+        return action
+
+    @APIParamsCall
+    def create_vnf_pm_job(self, body):
+        return self.post(
+            self.vnf_pm_jobs_path, body=body, headers=self.headers)
+
+    @APIParamsCall
+    def list_vnf_pm_jobs(self, retrieve_all=True, **_params):
+        vnf_pm_jobs = self.list(
+            "vnf_pm_jobs", self.vnf_pm_jobs_path, retrieve_all,
+            headers=self.headers, **_params)
+        return vnf_pm_jobs
+
+    @APIParamsCall
+    def show_vnf_pm_job(self, vnf_pm_job_id):
+        return self.get(
+            self.vnf_pm_job_path % vnf_pm_job_id, headers=self.headers)
+
+    @APIParamsCall
+    def update_vnf_pm_job(self, vnf_pm_job_id, body):
+        return self.patch(
+            self.vnf_pm_job_path % vnf_pm_job_id, body=body,
+            headers=self.headers)
+
+    @APIParamsCall
+    def delete_vnf_pm_job(self, vnf_pm_job_id):
+        return self.delete(
+            self.vnf_pm_job_path % vnf_pm_job_id, headers=self.headers)
+
+    @APIParamsCall
+    def show_vnf_pm_report(self, vnf_pm_job_id, vnf_pm_report_id):
+        return self.get(
+            self.vnf_pm_reports_path % {
+                'job_id': vnf_pm_job_id, 'report_id': vnf_pm_report_id
+            }, headers=self.headers)
 
 
 class Client(object):
@@ -940,7 +1164,10 @@ class Client(object):
     """
 
     def __init__(self, **kwargs):
-        self.vnf_lcm_client = VnfLCMClient(**kwargs)
+        api_version = kwargs.pop('api_version', '1')
+        self.vnf_lcm_client = VnfLCMClient(api_version, **kwargs)
+        self.vnf_fm_client = VnfFMClient(**kwargs)
+        self.vnf_pm_client = VnfPMClient(**kwargs)
         self.vnf_package_client = VnfPackageClient(**kwargs)
         self.legacy_client = LegacyClient(**kwargs)
 
@@ -1192,6 +1419,12 @@ class Client(object):
     def scale_vnf_instance(self, vnf_id, body):
         return self.vnf_lcm_client.scale_vnf_instance(vnf_id, body)
 
+    def change_ext_conn_vnf_instance(self, vnf_id, body):
+        return self.vnf_lcm_client.change_ext_conn_vnf_instance(vnf_id, body)
+
+    def change_vnfpkg_vnf_instance(self, vnf_id, body):
+        return self.vnf_lcm_client.change_vnfpkg_vnf_instance(vnf_id, body)
+
     def delete_vnf_instance(self, vnf_id):
         return self.vnf_lcm_client.delete_vnf_instance(vnf_id)
 
@@ -1201,6 +1434,15 @@ class Client(object):
     def rollback_vnf_instance(self, occ_id):
         return self.vnf_lcm_client.rollback_vnf_instance(occ_id)
 
+    def cancel_vnf_instance(self, occ_id, body):
+        return self.vnf_lcm_client.cancel_vnf_instance(occ_id, body)
+
+    def fail_vnf_instance(self, occ_id):
+        return self.vnf_lcm_client.fail_vnf_instance(occ_id)
+
+    def retry_vnf_instance(self, occ_id):
+        return self.vnf_lcm_client.retry_vnf_instance(occ_id)
+
     def update_vnf_package(self, vnf_package, body):
         return self.vnf_package_client.update_vnf_package(vnf_package, body)
 
@@ -1215,3 +1457,73 @@ class Client(object):
 
     def download_vnf_package(self, vnf_package):
         return self.vnf_package_client.download_vnf_package(vnf_package)
+
+    def list_vnf_lcm_op_occs(self, retrieve_all=True, **_params):
+        return self.vnf_lcm_client.list_vnf_lcm_op_occs(
+            retrieve_all=retrieve_all, **_params)
+
+    def show_vnf_lcm_op_occs(self, occ_id):
+        return self.vnf_lcm_client.show_vnf_lcm_op_occs(occ_id)
+
+    def create_lccn_subscription(self, body):
+        return self.vnf_lcm_client.create_lccn_subscription(body)
+
+    def delete_lccn_subscription(self, subsc_id):
+        return self.vnf_lcm_client.delete_lccn_subscription(subsc_id)
+
+    def list_lccn_subscriptions(self, retrieve_all=True, **_params):
+        return self.vnf_lcm_client.list_lccn_subscriptions(
+            retrieve_all=retrieve_all, **_params)
+
+    def show_lccn_subscription(self, subsc_id):
+        return self.vnf_lcm_client.show_lccn_subscription(subsc_id)
+
+    def show_vnf_lcm_versions(self, major_version):
+        return self.vnf_lcm_client.show_vnf_lcm_versions(major_version)
+
+    # VnfFMClient methods.
+
+    def list_vnf_fm_alarms(self, retrieve_all=True, **_params):
+        return self.vnf_fm_client.list_vnf_fm_alarms(
+            retrieve_all=retrieve_all, **_params)
+
+    def show_vnf_fm_alarm(self, vnf_fm_alarm_id):
+        return self.vnf_fm_client.show_vnf_fm_alarm(vnf_fm_alarm_id)
+
+    def update_vnf_fm_alarm(self, vnf_fm_alarm_id, body):
+        return self.vnf_fm_client.update_vnf_fm_alarm(vnf_fm_alarm_id, body)
+
+    def create_vnf_fm_sub(self, body):
+        return self.vnf_fm_client.create_vnf_fm_sub(body)
+
+    def list_vnf_fm_subs(self, retrieve_all=True, **_params):
+        return self.vnf_fm_client.list_vnf_fm_subs(
+            retrieve_all=retrieve_all, **_params)
+
+    def show_vnf_fm_sub(self, vnf_fm_sub_id):
+        return self.vnf_fm_client.show_vnf_fm_sub(vnf_fm_sub_id)
+
+    def delete_vnf_fm_sub(self, vnf_fm_sub_id):
+        return self.vnf_fm_client.delete_vnf_fm_sub(vnf_fm_sub_id)
+
+    # VnfPMClient methods.
+
+    def create_vnf_pm_job(self, body):
+        return self.vnf_pm_client.create_vnf_pm_job(body)
+
+    def list_vnf_pm_jobs(self, retrieve_all=True, **_params):
+        return self.vnf_pm_client.list_vnf_pm_jobs(
+            retrieve_all=retrieve_all, **_params)
+
+    def show_vnf_pm_job(self, vnf_pm_job_id):
+        return self.vnf_pm_client.show_vnf_pm_job(vnf_pm_job_id)
+
+    def update_vnf_pm_job(self, vnf_pm_job_id, body):
+        return self.vnf_pm_client.update_vnf_pm_job(vnf_pm_job_id, body)
+
+    def delete_vnf_pm_job(self, vnf_pm_job_id):
+        return self.vnf_pm_client.delete_vnf_pm_job(vnf_pm_job_id)
+
+    def show_vnf_pm_report(self, vnf_pm_job_id, vnf_pm_report_id):
+        return self.vnf_pm_client.show_vnf_pm_report(
+            vnf_pm_job_id, vnf_pm_report_id)
